# Write two (2) PDFs to a destination file
from PyPDF3 import PdfFileWriter, PdfFileReader


def overlay_pdfs(top_pdf, bottom_pdf, destination):
    """
    Overlay PDF objects to files
    :param top_pdf: PDF object to be placed on top
    :param bottom_pdf: PDF file to be placed underneath
    :param destination: Desintation path
    """
    drawing = PdfFileReader(top_pdf)  # Create new PDF object
    template = PdfFileReader(bottom_pdf)  # read your existing PDF

    # add the "watermark" (which is the new pdf) on the existing page
    page = template.getPage(0)
    page.mergePage(drawing.getPage(0))
    output = PdfFileWriter()  # Create new PDF file
    output.addPage(page)

    # finally, write "output" to a real file
    with open(destination, "wb") as outputStream:
        output.write(outputStream)


def write_pdf(pdf_obj, destination):
    """
    Write PDF object to file
    :param pdf_obj: PDF object to be written to file
    :param destination: Desintation path
    """
    reader = PdfFileReader(pdf_obj)  # Create new PDF object
    writer = PdfFileWriter()

    page_count = reader.getNumPages()

    # add the "watermark" (which is the new pdf) on the existing page
    for page_number in range(page_count):
        page = reader.getPage(page_number)
        writer.addPage(page)

    # finally, write "output" to a real file
    with open(destination, "wb") as outputStream:
        writer.write(outputStream)



# Dynamically generate watermark pdf file
import io
from tempfile import NamedTemporaryFile, mkdtemp
from reportlab.pdfgen.canvas import Canvas
from reportlab.pdfbase.pdfmetrics import stringWidth
from PyBundle import resource_path
from .image import img_opacity
from ..canvas import CanvasStr, CanvasImg
from pdf.utils import write_pdf
from pdf.conduit.lib import LETTER


def text_width(string, font_name, font_size):
    """Determine with width in pixels of string."""
    return stringWidth(string, fontName=font_name, fontSize=font_size)


def center_str(txt, font_name, font_size, offset=0):
    """Center a string on the x axis of a reportslab canvas"""
    return -(text_width(txt, font_name, font_size) / 2.0) + offset


def split_str(string):
    """Split string in half to return two strings"""
    split = string.split(' ')
    return ' '.join(split[:len(split) // 2]), ' '.join(split[len(split) // 2:])


class DrawPDF:
    def __init__(self, tempdir=None, compress=0, pagesize=LETTER):
        if tempdir:
            self.dir = tempdir
        else:
            self.dir = mkdtemp()
        tmppdf = NamedTemporaryFile(suffix='.pdf', dir=self.dir, delete=False)
        self.dst = resource_path(tmppdf.name)

        # create a new PDF with Reportlab
        self.packet = io.BytesIO()
        self.can = Canvas(self.packet, pagesize=pagesize, pageCompression=compress, bottomup=1)  # Initialize canvas

    def __str__(self):
        return str(self.dst)

    def _write(self):
        self.packet.seek(0)  # move to the beginning of the StringIO buffer
        write_pdf(self.packet, self.dst)  # Save new pdf file
        return self.dst

    def write(self):
        return self._write()


class WatermarkDraw(DrawPDF):
    def __init__(self, canvas_objects, rotate=0, compress=0, pagesize=LETTER, tempdir=None, pagescale=False):
        super(WatermarkDraw, self).__init__(tempdir, compress, pagesize)
        self.canvas_objects = canvas_objects
        self.rotate = rotate

        # Scale width, height and font size if pagesize is not letter sized
        if pagescale and self.can._pagesize != LETTER:
            w_scale = self.can._pagesize[0] / LETTER[0]
            h_scale = self.can._pagesize[1] / LETTER[1]
            for i in self.canvas_objects:
                if hasattr(i, 'w'):
                    i.w = i.w * w_scale
                if hasattr(i, 'h'):
                    i.h = i.h * h_scale
                if hasattr(i, 'size'):
                    i.size = i.size * ((h_scale * w_scale) / 2)

        self.draw()

    def draw(self):
        # Move canvas origin to the middle of the page
        self.can.translate(self.can._pagesize[0] / 2, self.can._pagesize[1] / 2)

        # Rotate canvas
        self.can.rotate(self.rotate)

        # Iterate canvas objects and determine if string or image
        for obj in self.canvas_objects:
            # Adjust x and y based on rotation
            obj.x += int(self.rotate * 1.5)
            obj.y += -int(self.rotate * 3)

            # Check if CanvasObject is a string or image
            if isinstance(obj, CanvasStr):
                self._draw_string(obj)
            elif isinstance(obj, CanvasImg):
                self._draw_image(obj)
        self.can.showPage()
        self.can.save()

    def _draw_image(self, ci):
        """
        Draw image object to reportlabs canvas.

        :param ci: CanvasImage object
        """
        img = img_opacity(ci.image, ci.opacity, tempdir=self.dir)
        self.can.drawImage(img, x=ci.x, y=ci.y, width=ci.w, height=ci.h, mask=ci.mask,
                           preserveAspectRatio=ci.preserve_aspect_ratio, anchorAtXY=True)

    def _draw_string(self, cs):
        """
        Draw string object to reportlabs canvas.

        Canvas Parameter changes (applied if set values differ from string object values)
        1. Font name
        2. Font size
        3. Font fill color & opacity
        4. X and Y position

        :param cs: CanvasString object
        """
        # 1. Font name
        if self.can._fontname != cs.font:
            self.can.setFont(cs.font, cs.size)

        # 2. Font size
        elif self.can._fontsize != cs.size:
            self.can.setFontSize(cs.size)

        # 3. Font file color
        self.can.setFillColor(cs.color, cs.opacity)

        # 4. X and Y positions
        # X and Y are both centered
        if cs.y_centered and cs.x_centered:
            # Check if text_width is greater than the canvas page width
            if text_width(cs.string, cs.font, cs.size) > self.can._pagesize[0]:
                str1, str2 = split_str(cs.string)
                self.can.drawString(x=center_str(str1, cs.font, cs.size, offset=0), y=cs.size, text=str1)
                self.can.drawString(x=center_str(str2, cs.font, cs.size, offset=0), y=-cs.size, text=str2)
                return
            else:
                x = center_str(cs.string, cs.font, cs.size, offset=0)
                y = 0

        # Y is centered and X is not
        elif cs.y_centered and not cs.x_centered:
            x = cs.x
            y = 0

        # X is centered and Y is not
        elif cs.x_centered and not cs.y_centered:
            x = center_str(cs.string, cs.font, cs.size, offset=0)
            y = cs.y
        else:
            x = cs.x
            y = cs.y
        self.can.drawString(x=x, y=y, text=cs.string)
        return



from datetime import datetime
from .objects import CanvasObjects, CanvasStr, CanvasImg
from ..draw.image import DrawPIL


class CanvasConstructor:
    def __init__(self, text1=None, text2=None, copyright=None, image=None, rotate=0, opacity=0.08, tempdir=None):
        self.text1 = text1
        self.text2 = text2
        self.copyright = copyright
        self.image = image
        self.rotate = rotate
        self.opacity = opacity
        self.tempdir = tempdir

        # Initialize CanvasObjects collector class and add objects
        self.obj = CanvasObjects()

    @property
    def objects(self):
        return self.obj, self.rotate

    def canvas(self):
        if self.image is not None:
            self.obj.add(CanvasImg(self.image, opacity=self.opacity, x=0, y=253))

            if self.text2 and self.copyright:
                self.obj.add(CanvasStr('© copyright ' + str(datetime.now().year), size=16, y=-170,
                                       opacity=self.opacity))
                self.obj.add(CanvasStr(self.text1, opacity=self.opacity, size=40, y=-360))
                self.obj.add(CanvasStr(self.text2, opacity=self.opacity, size=40, y=-560))
            elif self.text2 and not self.copyright:
                self.obj.add(CanvasStr(self.text1, opacity=self.opacity, size=40, y=90))
                self.obj.add(CanvasStr(self.text2, opacity=self.opacity, size=40, y=0))
            else:
                self.obj.add(CanvasStr(self.text1, opacity=self.opacity, size=40, y=-125))
        else:
            if self.text2 and self.copyright:
                self.obj.add(CanvasStr(self.text1, opacity=self.opacity, size=80, y=110))
                self.obj.add(CanvasStr(self.text2, opacity=self.opacity, size=80, y=20))
                self.obj.add(CanvasStr('© copyright ' + str(datetime.now().year), size=16, y=-30, opacity=self.opacity))
            elif self.text2 and not self.copyright:
                self.obj.add(CanvasStr(self.text1, opacity=self.opacity, size=80, y=90))
                self.obj.add(CanvasStr(self.text2, opacity=self.opacity, size=80, y=0))
            else:
                self.obj.add(CanvasStr(self.text1, opacity=self.opacity, size=40, y=50))
        return self.objects

    def img(self):
        img = DrawPIL(tempdir=self.tempdir)

        if self.image is not None:
            img.draw_img(self.image, x=0, y=50, opacity=self.opacity)
            if self.copyright:
                img.draw_text('© copyright ' + str(datetime.now().year), size=16, y='center')
            if self.text2:
                img.draw_text(self.text1, size=40, y=416, opacity=self.opacity)
                img.draw_text(self.text2, size=40, y=466, opacity=self.opacity)
            else:
                img.draw_text(self.text2, size=40, y=427, opacity=self.opacity)
        else:
            if self.text2 and self.copyright:
                img.draw_text('© copyright ' + str(datetime.now().year), size=16, y=350)
                img.draw_text(self.text1, size=80, y=106, opacity=self.opacity)
                img.draw_text(self.text2, size=80, y=206, opacity=self.opacity)
            elif self.text2 and not self.copyright:
                img.draw_text(self.text1, size=80, y=106, opacity=self.opacity)
                img.draw_text(self.text2, size=80, y=206, opacity=self.opacity)
            else:
                img.draw_text(self.text1, size=80, y='center', opacity=self.opacity)

        img.rotate(self.rotate)
        self.rotate = 0
        i = img.save()
        self.obj.add(CanvasImg(i, opacity=1, centered=True))
        return self.objects


from pdf.conduit.lib import LETTER


class CanvasStr:
    """Canvas string data object used for storing canvas.drawString parameters."""
    def __init__(self, string, font='Vera', color='black', size=40, opacity=0.1, x=0, y=0,
                 x_centered=True, y_centered=False):
        self.string = string
        self.font = font
        self.color = color
        self.size = size
        self.opacity = opacity
        self.x = x
        self.y = y
        self.x_centered = x_centered
        self.y_centered = y_centered


class CanvasImg:
    """Canvas image data object used for storing canvas.drawImage parameters."""
    def __init__(self, image, opacity=0.1, x=0, y=0, w=LETTER[0], h=LETTER[1], mask='auto',
                 preserve_aspect_ratio=True, centered=False):
        self.image = image
        self.opacity = opacity
        self.x = x
        self.y = y
        self.w = w
        self.h = h
        self.mask = mask
        self.preserve_aspect_ratio = preserve_aspect_ratio
        self.centered = centered


class CanvasObjects:
    """Canvas object collector to store list of canvas objects."""
    def __init__(self):
        self.objects = []

    def __iter__(self):
        return iter(self.objects)

    def add(self, canvas_object):
        self.objects.append(canvas_object)



# Extract images from a PDF
from PIL import Image
from pdf.utils.info import Info


# Todo: Fix img_extract and develop tests
def img_extract(pdf, password=None):
    # Read PDF file
    reader = Info(pdf, password).pdf

    # Number of pages in input document
    page_count = reader.getNumPages()

    # 5c. Go through all the input file pages to add a watermark to them
    for page_number in range(page_count):
        # Merge the watermark with the page
        page = reader.getPage(page_number)

        xobj = page['/Resources']['/XObject'].getObject()
        for obj in xobj:
            if xobj[obj]['/Subtype'] == '/Image':
                size = (xobj[obj]['/Width'], xobj[obj]['/Height'])
                data = xobj[obj].getData()
                if xobj[obj]['/ColorSpace'] == '/DeviceRGB':
                    mode = "RGB"
                else:
                    mode = "P"

                if xobj[obj]['/Filter'] == '/FlateDecode':
                    img = Image.frombytes(mode, size, data)
                    img.save(obj[1:] + ".png")  # TODO: Add save destination parameter
                elif xobj[obj]['/Filter'] == '/DCTDecode':
                    img = open(obj[1:] + ".jpg", "wb")
                    img.write(data)
                    img.close()
                elif xobj[obj]['/Filter'] == '/JPXDecode':
                    img = open(obj[1:] + ".jp2", "wb")
                    img.write(data)
                    img.close()


# TODO: Fix text extract and interpret extracted text
def text_extract(path, password=None):
    """Extract text from a PDF file"""
    pdf = Info(path, password).pdf

    return [pdf.getPage(i).extractText() for i in range(pdf.getNumPages())]



# Retrieve information about a PDF document
from PyPDF3 import PdfFileReader


class Info:
    def __init__(self, path, password=None, prompt=True):
        self.pdf = self._reader(path, password, prompt)

    @staticmethod
    def _reader(path, password, prompt):
        """Read PDF and decrypt if encrypted."""
        pdf = PdfFileReader(path) if not isinstance(path, PdfFileReader) else path
        # Check that PDF is encrypted
        if pdf.isEncrypted:
            # Check that password is none
            if not password:
                pdf.decrypt('')
                # Try and decrypt PDF using no password, prompt for password
                if pdf.isEncrypted and prompt:
                    print('No password has been given for encrypted PDF ', path)
                    password = input('Enter Password: ')
                else:
                    return False
            pdf.decrypt(password)
        return pdf

    @staticmethod
    def _resolved_objects(pdf, xobject):
        """Retrieve rotatation info."""
        return [pdf.getPage(i).get(xobject) for i in range(pdf.getNumPages())][0]

    @property
    def encrypted(self):
        """Check weather a PDF is encrypted"""
        return True if self.pdf.isEncrypted else False

    @property
    def decrypted(self):
        """Check weather a PDF is encrypted"""
        return True if self.pdf.isDecrypted else False

    @property
    def pages(self):
        """Retrieve PDF number of pages"""
        return self.pdf.getNumPages()

    @property
    def metadata(self):
        """Retrieve PDF metadata"""
        return self.pdf.getDocumentInfo()

    def resources(self):
        """Retrieve contents of each page of PDF"""
        return [self.pdf.getPage(i) for i in range(self.pdf.getNumPages())]

    @property
    def security(self):
        """Print security object information for a pdf document"""
        return {k: v for i in self.pdf.resolvedObjects.items() for k, v in i[1].items()}

    @property
    def dimensions(self):
        """Get width and height of a PDF"""
        size = self.pdf.getPage(0).mediaBox
        return {'w': float(size[2]), 'h': float(size[3])}

    @property
    def size(self):
        """Get width and height of a PDF"""
        size = self.pdf.getPage(0).mediaBox
        return float(size[2]), float(size[3])

    @property
    def rotate(self):
        """Retrieve rotation info."""
        return self._resolved_objects(self.pdf, '/Rotate')



__all__ = []


# Conduit installation
try:
    from pdf.conduit import Encrypt, Merge, Watermark, Label, WatermarkAdd
    CONDUIT_INSTALL = True
    __all__.extend(["Encrypt", "Merge", "Watermark", "Label", "WatermarkAdd"])
except ImportError:
    CONDUIT_INSTALL = False


# Utils installation
try:
    from pdf.utils.info import Info
    UTILS_INSTALL = True
    __all__.extend(["Info"])
except ImportError:
    UTILS_INSTALL = False


# GUI installation
try:
    from pdf.gui.gui import GUI
    GUI_INSTALLED = True
    __all__.extend(["GUI"])
except ImportError:
    GUI_INSTALLED = False


# Modify installation
try:
    from pdf.modify import upscale, rotate, slicer
    MODIFY_INSTALLED = True
    __all__.extend(["slicer", "upscale", "rotate"])
except ImportError:
    MODIFY_INSTALLED = False


# Convert installation
try:
    from pdf.convert.flatten import Flatten
    MODIFY_INSTALLED = True
    __all__.extend(["Flatten"])
except ImportError:
    MODIFY_INSTALLED = False



# Merge PDF documents
import os
from PyPDF3 import PdfFileMerger
from pdfrw import PdfReader, PdfWriter, IndirectPdfDict


class Merge:
    def __init__(self, input_pdfs, output_name='merged', output_dir=None, method='pdfrw'):
        self.pdfs = self._get_pdf_list(input_pdfs)
        self.directory = output_dir if output_dir else os.path.dirname(self.pdfs[0])
        self.output = os.path.join(self.directory, output_name + '.pdf')
        self.method = method
        self.file = self.merge(self.pdfs, self.output)

    def __str__(self):
        return str(self.file)

    @staticmethod
    def validate(pdf):
        if not pdf.startswith('.') and pdf.endswith('.pdf'):
            return True

    def _get_pdf_list(self, input_pdfs):
        """
        Generate list of PDF documents.

        :param input_pdfs: List of PDFs or a directory path
             Directory - Scans directory contents
             List - Filters list to assert all list items are paths to PDF documents
        :return: List of PDF paths
        """
        if type(input_pdfs) is list:
            return [pdf for pdf in input_pdfs if self.validate(pdf)]
        elif os.path.isdir(input_pdfs):
            return [os.path.join(input_pdfs, pdf) for pdf in os.listdir(input_pdfs) if self.validate(pdf)]

    def merge(self, pdf_files, output):
        """Merge list of PDF files to a single PDF file."""
        if self.method is 'pypdf3':
            return self.pypdf3(pdf_files, output)
        else:
            return self.pdfrw(pdf_files, output)

    @staticmethod
    def pypdf3(pdf_files, output):
        # Create PDF file merger object
        pdf_merger = PdfFileMerger()

        # Appending pdfs one by one
        for pdf in pdf_files:
            pdf_merger.append(pdf)

        # writing combined pdf to output pdf file
        with open(output, 'wb') as f:
            pdf_merger.write(f)
        return output

    @staticmethod
    def pdfrw(pdf_files, output):
        writer = PdfWriter()
        for inpfn in pdf_files:
            writer.addpages(PdfReader(inpfn).pages)

        writer.trailer.Info = IndirectPdfDict(
            Title='HPA Design',
            Author='HPA Design',
            Subject='HPA Design',
            Creator='HPA Design',
        )
        writer.write(output)
        return output


def main():
    from pdf.gui.gui import get_directory
    d = get_directory()
    m = Merge(d)
    print(m.output)


if __name__ == '__main__':
    main()



# Set directory paths and file names
import os
import sys
from pathlib import Path
from PyBundle import bundle_dir


if 'pathlib' in sys.modules:
    def _add_suffix(file_path, suffix, sep, ext):
        p = Path(file_path)
        _ext = p.suffix if ext is None else str('.' + ext.strip('.'))
        out = p.stem + sep + suffix + _ext  # p.suffix is file extension
        return os.path.join(os.path.dirname(file_path), out)
else:
    def _add_suffix(file_path, suffix, sep, ext):
        split = os.path.basename(file_path).rsplit('.', 1)
        ext = split[1] if ext is None else str('.' + ext.strip('.'))
        return os.path.join(os.path.dirname(file_path), split[0] + sep + suffix + '.' + ext)


def add_suffix(file_path, suffix='modified', sep='_', ext=None):
    """Adds suffix to a file name seperated by an underscore and returns file path."""
    return _add_suffix(file_path, suffix, sep, ext)


def set_destination(source, suffix, filename=False, ext=None):
    """Create new pdf filename for temp files"""
    source_dirname = os.path.dirname(source)

    # Do not create nested temp folders (/temp/temp)
    if not source_dirname.endswith('temp'):
        directory = os.path.join(source_dirname, 'temp')  # directory
    else:
        directory = source_dirname

    # Create temp dir if it does not exist
    if not os.path.isdir(directory):
        os.mkdir(directory)

    # Parse source filename
    if filename:
        src_file_name = filename
    else:
        src_file_name = Path(source).stem  # file name
    if ext:
        src_file_ext = ext
    else:
        src_file_ext = Path(source).suffix  # file extension

    # Concatenate new filename
    dst_path = src_file_name + '_' + suffix + src_file_ext
    full_path = os.path.join(directory, dst_path)  # new full path

    if not os.path.exists(full_path):
        return full_path
    else:
        # If file exists, increment number until filename is unique
        number = 1
        while True:
            dst_path = src_file_name + '_' + suffix + '_' + str(number) + src_file_ext
            if not os.path.exists(dst_path):
                break
            number = number + 1
        full_path = os.path.join(directory, dst_path)  # new full path
        return full_path



import os
from datetime import datetime
import PySimpleGUI as sg


class Receipt:
    def __init__(self, use=True, gui=False):
        self.dst = None
        self.use = use
        self.gui = gui
        self.items = []
        self._print = self._gui_print if self.gui else print
        self.add('PDF Watermarker', datetime.now().strftime("%Y-%m-%d %H:%M"))

    @staticmethod
    def _gui_print(msg):
        sg.Print(msg)

    def set_dst(self, doc, file_name='watermark receipt.txt'):
        self.dst = os.path.join(os.path.dirname(doc), file_name)
        self.add('Directory', os.path.dirname(doc))
        self.add('PDF', os.path.basename(doc))
        return self

    def add(self, key, value):
        message = str("{0:20}--> {1}".format(key, value))
        if self.use:
            self._print(message)
        self.items.append(message)

    def dump(self):
        exists = os.path.isfile(self.dst)
        with open(self.dst, 'a') as f:
            if exists:
                f.write('*******************************************************************\n')

            for item in self.items:
                f.write(item + '\n')

        if self.gui:
            sg.Popup('Success!')



import os
from tempfile import NamedTemporaryFile
from PIL import Image, ImageEnhance, ImageDraw, ImageFont
from PyBundle import bundle_dir, resource_path
from pdf.conduit.lib import FONT


def img_opacity(image, opacity, tempdir=None, bw=True):
    """
    Reduce the opacity of a PNG image.

    Inspiration: http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/362879

    :param image: PNG image file
    :param opacity: float representing opacity percentage
    :param tempdir: Temporary directory
    :param bw: Set image to black and white
    :return:  Path to modified PNG
    """
    # Validate parameters
    try:
        assert 0 <= opacity < 1
    except AssertionError:
        return image
    assert os.path.isfile(image), 'Image is not a file'

    # Open image in RGBA mode if not already in RGBA
    with Image.open(image) as im:
        if im.mode != 'RGBA':
            im = im.convert('RGBA')
        else:
            im = im.copy()

        # Adjust opacity
        alpha = im.split()[3]
        alpha = ImageEnhance.Brightness(alpha).enhance(opacity)
        im.putalpha(alpha)
        if bw:
            im.convert('L')

        # Save modified image file
        dst = NamedTemporaryFile(suffix='.png', dir=tempdir, delete=False).name
        im.save(dst)
    return dst


class DrawPIL:
    def __init__(self, size=(792, 612), tempdir=None):
        # Create a black image
        self.img = Image.new('RGBA', size, color=(255, 255, 255, 0))  # 2200, 1700 for 200 DPI
        self.tempdir = tempdir

    def _centered_x(self, text, drawing, font):
        # Get img size
        page_width = self.img.size[0]
        text_width = drawing.textsize(text, font=font)[0]
        x = (page_width - text_width) / 2
        return x

    def _centered_y(self, size):
        # Get img size
        page_height = self.img.size[1]
        y = (page_height / 2) - size
        return y

    def _scale(self, img, func='min'):
        im = img if isinstance(img, Image.Image) else Image.open(img)

        if func is 'min':
            scale = min(float(self.img.size[0] / im.size[0]), float(self.img.size[1] / im.size[1]))
        else:
            scale = max(float(self.img.size[0] / im.size[0]), float(self.img.size[1] / im.size[1]))

        im.thumbnail((int(im.size[0] * scale), int(im.size[1] * scale)))

        return im if isinstance(img, Image.Image) else self.save(img=im)

    def draw_text(self, text, x='center', y=140, font=FONT, size=40, opacity=0.1):
        # Set drawing context
        d = ImageDraw.Draw(self.img)

        # Set a font
        fnt = ImageFont.truetype(font, int(size * 1.00))  # multiply size of font if needed

        # Check if x is set to 'center'
        if 'center' in str(x).lower():
            x = self._centered_x(text, d, fnt)

        # Check if y is set to 'center'
        if 'center' in str(y).lower():
            y = self._centered_y(size)

        # Draw text to image
        d.text((x, y), text, font=fnt, fill=(0, 0, 0, int(255 / (opacity * 100))))

    def draw_img(self, img, x=0, y=0, opacity=1.0):
        scaled = self._scale(img)
        opacity = Image.open(img_opacity(scaled, opacity, self.tempdir))
        self.img.paste(opacity, (x, y))

    def rotate(self, rotate):
        # Create transparent image that is the same size as self.img
        mask = Image.new('L', self.img.size, 255)

        # Rotate image and then scale image to fit self.img
        front = self.img.rotate(rotate, expand=True)

        # Rotate mask
        mask.rotate(rotate, expand=True)

        # Determine difference in size between mask and front
        y_margin = int((mask.size[1] - front.size[1]) / 3)

        # Create another new image
        rotated = Image.new('RGBA', self.img.size, color=(255, 255, 255, 0))

        # Paste front into new image and set x offset equal to half
        # the difference of front and mask size
        rotated.paste(front, (0, y_margin))
        self.img = rotated

    def save(self, img=None, destination=None, file_name='pil'):
        img = self.img if not img else img
        fn = file_name.strip('.png') if '.png' in file_name else file_name
        if self.tempdir:
            tmpimg = NamedTemporaryFile(suffix='.png', dir=self.tempdir, delete=False)
            output = resource_path(tmpimg.name)
        elif destination:
            output = os.path.join(destination, fn + '.png')
        else:
            output = os.path.join(bundle_dir(), fn + '.png')

        # Save image file
        img.save(output)
        return output



import os
from reportlab.pdfbase import pdfmetrics
from reportlab.pdfbase.ttfonts import TTFont
from reportlab.lib.pagesizes import letter
from PyBundle import bundle_dir, resource_path


def register_font(font='Vera.ttf'):
    """Register fonts for report labs canvas."""
    directory = os.path.join(bundle_dir(), 'font')
    ttfFile = resource_path(os.path.join(directory, font))
    if os.path.exists(ttfFile):
        pdfmetrics.registerFont(TTFont("Vera", ttfFile))
        return ttfFile
    else:
        print(ttfFile, 'can not be found')


def _image_directory():
    directory = os.path.join(bundle_dir(), 'img')
    if os.path.exists(directory):
        return directory
    else:
        print(directory, 'can not be found')


IMAGE_DIRECTORY = _image_directory()


def available_images():
    imgs = [i for i in os.listdir(IMAGE_DIRECTORY) if not i.startswith('.')]
    if len(imgs) > 0:
        return sorted(imgs, reverse=True)
    else:
        return ['Add images...']


FONT = register_font()
LETTER = letter[1], letter[0]
IMAGE_DEFAULT = resource_path('Wide.png')



import re
from setuptools import setup, find_packages

long_description = """
A Pure-Python library built as a PDF toolkit.  Prepare documents for distribution.

Features:
- Watermark: Dynamically generate watermarks and add watermark to existing document
- Label: Overlay text labels such as filename or date to documents 
- Encrypt: Password protect and restrict permissions to print only
- Rotate: Rotate by increments of 90 degrees
- Upscale: Scale PDF size
- Merge: Concatenate multiple documents into one file
- Slice: Extract page ranges from documents
- Extract Text and Images
- Retrieve document metadata and information
"""

# Retrieve version number
VERSIONFILE = "pdf/conduit/_version.py"
verstrline = open(VERSIONFILE, "rt").read()
VSRE = r"^__version__ = ['\"]([^'\"]*)['\"]"
mo = re.search(VSRE, verstrline, re.M)
if mo:
    verstr = mo.group(1)
else:
    raise RuntimeError("Unable to find version string in %s." % (VERSIONFILE))

setup(
    install_requires=[
        'PyPDF3>=1.0.1',
        'pdfrw',
        'Pillow',
        'reportlab',
        'PyBundle>=1.0.5',
        'looptools',
        'pdfconduit-utils',
        'pdfconduit-modify'
    ],
    name='pdfconduit',
    version=verstr,
    packages=find_packages(),
    namespace_packages=['pdf'],
    include_package_data=True,
    url='https://github.com/mrstephenneal/pdfconduit',
    license='',
    author='Stephen Neal',
    author_email='stephen@stephenneal.net',
    description='PDF toolkit for preparing documents for distribution.',
    long_description=long_description,
)



# Generate sample PDF documents
import os
from pdf.gui.gui import available_images
from pdf.conduit import Label, Watermark, Merge
from pdf.utils import Info
from pdf.modify import slicer, upscale
from pdf.convert import Flatten
from tests import pdf


class Samples:
    def __init__(self, src, dst):
        if not os.path.exists(dst):
            os.mkdir(dst)
        self.src = src
        self.dst = dst
        self.wm = Watermark(self.src, use_receipt=False, open_file=False, remove_temps=True)

    def _title(self, document, title='PDF Samples'):
        return Label(document, title, title_page=True, tempdir=self.wm.tempdir).watermark

    def cleanup(self):
        self.wm.cleanup()

    def watermarks(self, images=available_images()):
        watermarks = []
        for i in images:
            wm = self.wm.draw(text1=i, image=i, copyright=False)
            watermarks.append(wm)
        watermarks.insert(0, self._title(wm, 'Watermark Images'))
        m = Merge(watermarks, 'Watermarks samples', self.dst)
        return m.file

    def opacity(self):
        samples = []
        _range = range(4, 25)[::3]
        if Info(self.src).pages > 1:
            self.src = slicer(self.src, 1, 1, tempdir=self.wm.tempdir)
        for i in _range:
            o = i * .01
            wtrmrk = self.wm.draw(text1='200 Stonewall Blvd', text2='Wrentham, MA', opacity=o)
            watermarked = self.wm.add(document=self.src, watermark=wtrmrk)

            labeled_pdf = Label(watermarked, str(str(i).zfill(2) + '%'), tempdir=self.wm.tempdir).write(cleanup=False)
            samples.append(labeled_pdf)
            samples.insert(0, self._title(labeled_pdf, 'Opacity Comparisons'))
        m = Merge(samples, 'Opacity comparison samples', self.dst)
        return m.file

    def placement(self):
        if Info(self.src).pages > 2:
            self.src = slicer(self.src, 1, 1, tempdir=self.wm.tempdir)
        wtrmrk = self.wm.draw(text1='200 Stonewall Blvd', text2='Wrentham, MA')
        over = self.wm.add(document=self.src, watermark=wtrmrk, underneath=False)
        over_with_label = Label(over, 'Overlayed watermark', suffix=None,
                                tempdir=self.wm.tempdir).write(cleanup=False)

        under = self.wm.add(document=self.src, watermark=wtrmrk, underneath=True)
        under_with_label = Label(under, 'Underneath watermarked', suffix=None,
                                 tempdir=self.wm.tempdir).write(cleanup=False)

        to_merge = [self._title(under, 'Watermark Placement'), over_with_label, under_with_label]
        m = Merge(to_merge, 'Watermark Placement samples', self.dst)
        return m.file

    def layering(self):
        if Info(self.src).pages > 2:
            self.src = slicer(self.src, 1, 1, tempdir=self.wm.tempdir)
        flat = self.wm.draw(text1='200 Stonewall Blvd', text2='Wrentham, MA', flatten=True)
        layered = self.wm.draw(text1='200 Stonewall Blvd', text2='Wrentham, MA', flatten=False)

        wtrmrked_flat = Label(self.wm.add(document=self.src, watermark=flat), 'Flat watermark',
                              tempdir=self.wm.tempdir).write(cleanup=False)
        wtrmrked_layer = Label(self.wm.add(document=self.src, watermark=layered), 'Layered watermark',
                               tempdir=self.wm.tempdir).write(cleanup=False)

        watermark_flat = Label(flat, 'Flat watermark', tempdir=self.wm.tempdir).write(cleanup=False)
        watermark_layer = Label(layered, 'Layered watermark', tempdir=self.wm.tempdir).write(cleanup=False)

        to_merge = [self._title(watermark_layer, 'Watermark Layering'), wtrmrked_flat, watermark_flat, wtrmrked_layer, watermark_layer]
        m = Merge(to_merge, 'Layering samples', self.dst)
        return m.file

    def flat(self):
        # Create one page PDF document
        if Info(self.src).pages > 2:
            self.src = slicer(self.src, 1, 1, tempdir=self.wm.tempdir)

        # Standard watermark and layering
        wtrmrk = self.wm.draw(text1='200 Stonewall Blvd', text2='Wrentham, MA')
        wtrmrked = self.wm.add(document=self.src, watermark=wtrmrk)
        wtrmrked_upscaled = upscale(wtrmrked, scale=2.0, tempdir=self.wm.tempdir)
        wtrmrked_labeled = Label(wtrmrked_upscaled, 'Layered PDF page', tempdir=self.wm.tempdir).write(cleanup=False)

        # Flattened layering
        flattened = Flatten(wtrmrked, tempdir=self.wm.tempdir, progress_bar='tqdm').save(remove_temps=False)
        flattened_labeled = Label(flattened, 'Flattened PDF page', tempdir=self.wm.tempdir).write(cleanup=False)

        # Merge files
        to_merge = [self._title(flattened_labeled, 'Flat vs. Layered pages'), flattened_labeled, wtrmrked_labeled]
        m = Merge(to_merge, 'Flat vs. Layered', self.dst)
        return m.file


def main():
    src = pdf
    dst = os.path.join(os.path.dirname(src), 'samples')

    s = Samples(src, dst)
    s.opacity()
    s.watermarks()
    s.placement()
    s.layering()
    s.flat()

    s.cleanup()


if __name__ == '__main__':
    main()



__all__ = ['set_destination', 'add_suffix', 'open_window', 'overlay_pdfs', 'write_pdf', 'Info', 'Receipt']


from .info import Info
from .path import set_destination, add_suffix
from .view import open_window
from .write import overlay_pdfs, write_pdf
from .receipt import Receipt



from pdf.conduit.encrypt import Encrypt
from pdf.conduit.merge import Merge
from pdf.conduit.watermark import WatermarkAdd, Watermark, Label


__all__ = ["Encrypt", "Merge", "Watermark", "Label", "WatermarkAdd"]



# View directories and files in a window
from sys import modules
from pathlib import Path
from subprocess import call, Popen


def open_window(path):
    """Open path in finder or explorer window"""
    if 'pathlib' in modules:
        try:
            call(["open", "-R", str(Path(str(path)))])
        except FileNotFoundError:
            Popen(r'explorer /select,' + str(Path(str(path))))
    else:
        print('pathlib module must be installed to execute open_window function')


# Encrypt a PDF file with password protection
from pdf.utils.path import add_suffix
from PyPDF3 import PdfFileReader, PdfFileWriter


class Encrypt:
    def __init__(self, pdf, user_pw, owner_pw=None, output=None, suffix='secured', bit128=True, allow_printing=True,
                 allow_commenting=False, overwrite_permission=None, progress_bar_enabled=False, progress_bar='gui'):
        """Password protect PDF file and allow all other permissions."""
        self.pdf = pdf
        self.user_pw = user_pw
        self.owner_pw = owner_pw
        self.output = add_suffix(pdf, suffix=suffix) if not output else output
        self.encrypt_128 = bit128
        self.allow_printing = allow_printing
        self.allow_commenting = allow_commenting
        self.overwrite_permission = overwrite_permission
        self.progress_bar_enabled = progress_bar_enabled
        self.progress_bar = progress_bar

        self.encrypt()

    def __str__(self):
        return str(self.output)

    def encrypt(self):
        # Create PDF writer object
        pdf_writer = PdfFileWriter()
        with open(self.pdf, 'rb') as pdf_file:
            # Read opened PDF file
            pdf_reader = PdfFileReader(pdf_file)

            # Add each page from source PDF
            for page_num in range(pdf_reader.numPages):
                page = pdf_reader.getPage(page_num)
                pdf_writer.addPage(page)

            # Apply encryption to writer object
            pdf_writer.encrypt(self.user_pw, self.owner_pw, use_128bit=self.encrypt_128,
                               allow_printing=self.allow_printing, allow_commenting=self.allow_commenting,
                               overwrite_permission=self.overwrite_permission)

            pdf_writer.addMetadata({
                '/Producer': 'pdf',
                '/Creator': 'HPA Design',
                '/Author': 'HPA Design',
            })

            # Write encrypted PDF to file
            with open(self.output, 'wb') as output_pdf:
                pdf_writer.write(output_pdf, progress_bar=self.progress_bar,
                                 progress_bar_enabled=self.progress_bar_enabled)
        return self.output


def main():
    from pdf.gui.gui import GUI
    GUI.encrypt()


if __name__ == '__main__':
    main()
